varying highp vec2 vTexCoord;

uniform highp sampler2D sTexture;//纹理图片

uniform lowp float brightness;//亮度值
uniform lowp float saturation;//饱和度
uniform lowp float contrast;//对比度
uniform lowp float hue;//色调
uniform lowp float temperature;//温度
uniform lowp float tint;
//暗角效果属性
uniform lowp vec2 vignetteCenter;
uniform lowp float vignetteColor;
uniform highp float vignetteStart;
uniform highp float vignetteEnd;

uniform highp float exposure; //曝光度
uniform highp float sharpness; //锐化

const mediump vec3 luminanceWeighting = vec3(0.2125, 0.7154, 0.0721);

const lowp vec3 warmFilter = vec3(0.93, 0.54, 0.0);
const mediump mat3 RGBtoYIQ = mat3(0.299, 0.587, 0.114, 0.596, -0.274, -0.322, 0.212, -0.523, 0.311);
const mediump mat3 YIQtoRGB = mat3(1.0, 0.956, 0.621, 1.0, -0.272, -0.647, 1.0, -1.105, 1.702);

highp vec3 rgb2hsv(highp vec3 c){
    highp vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    highp vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    highp vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));
    highp float d = q.x - min(q.w, q.y);
    highp float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}
highp vec3 hsv2rgb(highp vec3 c){
    highp vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    highp vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}

void main() {

    lowp vec4 textureColor = texture2D(sTexture, vTexCoord);

    //设置锐化开始
    vec4 color = vec4(0.0);
    int coreSize = 3;
    float texelOffset = 1.0 / 300.0;
    float kernel[9];

    kernel[6] = 0.0; kernel[7] = -1.0; kernel[8] = 0.0;
    kernel[3] = -1.0; kernel[4] = 4.0; kernel[5] = -1.0;
    kernel[0] = 0.0; kernel[1] = -1.0; kernel[2] = 0.0;
    int index = 0;
    for(int y = 0; y<coreSize; y++)
    {
        for(int x=0; x<coreSize; x++){
            vec2 tV = vec2(texelOffset*(float(x)-1.0),texelOffset*(float(y)-1.0));
            vec4 currentColor=texture2D(sTexture, vTexCoord+tV);
            color += currentColor * kernel[index++];
        }
    }

    //设置暗角开始
    lowp vec3 vignetteRgb = textureColor.rgb;
    lowp float vignetteD = distance(vTexCoord, vec2(0.5, 0.5));
    lowp float percent = smoothstep(vignetteStart, 1.0, vignetteD);
    lowp vec3 vignetteV3 = vec3(mix(vignetteRgb.x, vignetteColor, percent), mix(vignetteRgb.y, vignetteColor, percent), mix(vignetteRgb.z, vignetteColor, percent));

    //设置对比度开始
    lowp vec3 contrastV3 = vec3((vignetteV3 - vec3(0.5)) * contrast + vec3(0.5));
    //设置饱和度开始
    lowp float luminance = dot(contrastV3, luminanceWeighting);
    lowp vec3 greyScaleColor = vec3(luminance);
    lowp vec3 saturationV3 = mix(greyScaleColor, contrastV3, saturation);

    //设置色温开始
    mediump vec3 yiq = RGBtoYIQ * saturationV3; //adjusting tint
    yiq.b = clamp(yiq.b + tint*0.5226*0.1, -0.5226, 0.5226);
    lowp vec3 rgb = YIQtoRGB * yiq;
    lowp vec3 processed = vec3(
        (rgb.r < 0.5 ? (2.0 * rgb.r * warmFilter.r) : (1.0 - 2.0 * (1.0 - rgb.r) * (1.0 - warmFilter.r))), //adjusting temperature
        (rgb.g < 0.5 ? (2.0 * rgb.g * warmFilter.g) : (1.0 - 2.0 * (1.0 - rgb.g) * (1.0 - warmFilter.g))),
        (rgb.b < 0.5 ? (2.0 * rgb.b * warmFilter.b) : (1.0 - 2.0 * (1.0 - rgb.b) * (1.0 - warmFilter.b))));
    lowp vec3 temperatureV3 = mix(rgb, processed, temperature);

    //设置色调开始
    lowp vec3 hsl = rgb2hsv(temperatureV3.xyz);
    if(hue != 0.0)hsl.x = hue;
    if(hsl.x<0.0)hsl.x = hsl.x+1.0;
    else if(hsl.x>1.0)hsl.x = hsl.x-1.0;
    lowp vec3 hueV3 = hsv2rgb(hsl);

    gl_FragColor = sharpness * color + vec4(hueV3*brightness*pow(2.0, exposure), 1.0);
}